@@ -10,7 +10,16 @@ if {$envlibdir != {}} {
    set XCIRCUIT_LIB_DIR $path
 }
 unset envlibdir
-if {[catch [load ${path}/xcircuit.so]]} {load ${path}/tcl/xcircuit.so}
+
+set libfile [glob -nocomplain ${path}/xcircuit.so]
+if {$libfile == {}} {
+   set libfile [glob -nocomplain ${path}/tcl/xcircuit.so]
+}
+if {$libfile == {}} {
+   error "Library file xcircuit.so not found in $path"
+}
+load ${libfile}
+unset libfile
 
 package require Xcircuit
 
@@ -129,8 +138,6 @@ proc irsim { args } {
 
 proc magic { args } {
    global CAD_HOME
-   global argv
-   global argc
    set magicscript [glob -nocomplain ${CAD_HOME}/lib/magic/tcl/magic.tcl]
    if { ${magicscript} == {} } {
       puts stderr "\"magic\" requires Tcl-based Magic version 7.2 or newer."
@@ -148,8 +155,6 @@ proc magic { args } {
 
 proc netgen { args } {  
    global CAD_HOME
-   global argv
-   global argc
    set netgenscript [glob -nocomplain ${CAD_HOME}/lib/netgen/tcl/netgen.tcl]
    if { ${netgenscript} == {} } {
       puts stderr "\"netgen\" requires Tcl-based Netgen version 1.2 or newer."
@@ -159,7 +164,7 @@ proc netgen { args } {
    } else {
       set argv $args
       set argc [llength $args]
-      uplevel #0 source $netgenscript
+      source $netgenscript
    }
 }
 
