@@ -47,6 +47,10 @@ extern char _STR[150];
 extern short fontcount;
 extern fontinfo *fonts;
 
+/*------------------------------------------------------*/
+/* Check if a library exists				*/
+/*-------------------------------------------------------*/
+
 /*---------------------------------------------------------*/
 /* Find the Helvetica font for use in labeling the objects */
 /*---------------------------------------------------------*/
@@ -87,7 +91,7 @@ void catreturn()
 {
    /* Pop the object being edited from the push stack. */
 
-   popobject(NULL, NULL, NULL);
+   popobject(NULL, (pointertype)1, NULL);
 }
 
 /*------------------------------------------------------*/
@@ -273,6 +277,22 @@ int is_library(objectptr thisobj)
    return -1;
 }
 
+/*------------------------------------------------------*/
+/* Check if a library (given by name) exists.  If it	*/
+/* does, return the library number.  If not, return -1	*/
+/*------------------------------------------------------*/
+
+int check_library(char *libname)
+{
+   int i;
+   
+   for (i = 0; i < xobjs.numlibs; i++)
+      if (!strcmp(xobjs.libtop[i + LIBRARY]->thisobject->name, libname))
+	 return i;
+
+   return -1;
+}
+
 /*------------------------------------------------------*/
 /* ButtonPress handler during page catalog viewing mode */
 /*------------------------------------------------------*/
@@ -290,7 +310,6 @@ void pagecatbutton(XButtonEvent *event)
    if (event->button == Button1 || event->button == Button2) {
       if ((bpage = pageposition(mode, event, 0)) >= 0) {
 		  
-#ifdef SCHEMA
 	 if (eventmode == ASSOC_MODE) {
 	    if (mode == PAGELIB) {
 	       /* using changepage() allows use of new page for schematic */
@@ -306,12 +325,10 @@ void pagecatbutton(XButtonEvent *event)
 	    }
 	    return;
          }
-	 else
-#endif
-	 if (event->button == Button1) {
+	 else if (event->button == Button1) {
 
 	    /* like catreturn(), but don't actually go to the popped page */
-	    objectdeselect();
+	    unselect_all();
 	    eventmode = NORMAL_MODE;
 	    if (mode == PAGELIB) {
 	       newpage(bpage);
@@ -324,7 +341,7 @@ void pagecatbutton(XButtonEvent *event)
 	 }
 	 else {  /* Button2 */
 	    if (mode == PAGELIB)    /* No such method for LIBLIB is defined. */
-	       objectselect(OBJECT);
+	       select_add_element(OBJINST);
 	 }
       }
    }
@@ -382,8 +399,8 @@ void pageinstpos(short mode, short tpage, objinstptr drawinst, int gxsize,
 objinstptr newpageinst(objectptr pageobj)
 {
    objinstptr newinst = (objinstptr) malloc(sizeof(objinst));
-   objectdefaults(newinst, pageobj, 0, 0);
-   newinst->type = OBJECT;
+   instancedefaults(newinst, pageobj, 0, 0);
+   newinst->type = OBJINST;
    newinst->color = DEFAULTCOLOR;
    return newinst;
 }
@@ -428,7 +445,7 @@ void composepagelib(short mode)
    /* from the list.							 */
 
    for (pgen = directory->plist; pgen < directory->plist + directory->parts; pgen++)
-      if ((*pgen)->type == OBJECT) *pgen = NULL;
+      if (IS_OBJINST(*pgen)) *pgen = NULL;
 
    reset(directory, NORMAL);
 
@@ -530,7 +547,7 @@ void updatepagelib(short mode, short tpage)
 
    for (i = 0; i < libinst->parts; i++) {
       gelem = libinst->plist + i;
-      if ((*gelem)->type == OBJECT) {
+      if (IS_OBJINST(*gelem)) {
 	 pinst = TOOBJINST(gelem);
          if (pinst->thisobject == compobj) {
 	    /* recalculate scale and position of the object instance */
@@ -613,7 +630,7 @@ void pagecatmove(XKeyEvent *event)
       }
    }
 
-   objectdeselect();
+   unselect_all();
    composelib(PAGELIB);
    drawarea(NULL, NULL, NULL);
 }
@@ -645,7 +662,7 @@ void composelib(short mode)
    /* call reset(), we find the pointer to the instance and NULL it.	*/
    
    for (pgen = libpage->plist; pgen < libpage->plist + libpage->parts; pgen++)
-      if ((*pgen)->type == OBJECT) *pgen = NULL;
+      if (IS_OBJINST(*pgen)) *pgen = NULL;
 
    reset(libpage, NORMAL);
 
@@ -754,7 +771,7 @@ short finddepend(objinstptr libobj, objectptr **compobjp)
 		     
          for (testobj = (*compobj)->plist; testobj < (*compobj)->plist
 	          + (*compobj)->parts; testobj++) {
-	    if ((*testobj)->type == OBJECT) {
+	    if (IS_OBJINST(*testobj)) {
 	       if (TOOBJINST(testobj)->thisobject == libobj->thisobject) return 2;
 	    }
 	 }
@@ -769,7 +786,7 @@ short finddepend(objinstptr libobj, objectptr **compobjp)
       *compobjp = compobj;
       for (testobj = (*compobj)->plist; testobj < (*compobj)->plist
 	       + (*compobj)->parts; testobj++) {
-	 if ((*testobj)->type == OBJECT) {
+	 if (IS_OBJINST(*testobj)) {
 	    if (TOOBJINST(testobj)->thisobject == libobj->thisobject) return 1;
 	 }
       }
@@ -904,24 +921,14 @@ void catdelete()
       }
       else { 		/* All's clear to delete.		     	   */
 
-         /* First remove any instances of this object in the delete buffer */
+         /* Clear the undo stack so that any references to this object	*/
+	 /* won't screw up the database (this could be kinder & gentler	*/
+	 /* by first checking if there are any references to the object	*/
+	 /* in the undo stack. . .					*/
 
-         for (compobj = xobjs.delbuffer.library; compobj != xobjs.delbuffer.library +
-                xobjs.delbuffer.number; compobj++) {
-	    for (testobj = (*compobj)->plist; testobj < (*compobj)->plist
-	          + (*compobj)->parts; testobj++) {
-	       if ((*testobj)->type == OBJECT) {
-	          if (TOOBJINST(testobj)->thisobject == libobj->thisobject) {
-		     free(TOOBJINST(testobj));
-		     for (tobj = testobj; tobj < (*compobj)->plist
-			   + (*compobj)->parts - 1; tobj++) {
-		        (*tobj) = (*(tobj + 1));
-		     }
-		     (*compobj)->parts--;
-		  }
-	       }
-	    }
-         }
+	 flush_undo_stack();
+
+	 /* Next, remove the object from the library page. */
 
 	 for (tlib = libpage; tlib < libpage + *libpobjs; tlib++)
 	    if ((*tlib) == libobj->thisobject) {
@@ -1116,7 +1123,7 @@ void catmove(XKeyEvent *event)
 	 else if (areastruct.save.y <=
 		xobjs.libtop[i + LIBRARY]->thisobject->bbox.lowerleft.y +
 		xobjs.libtop[i + LIBRARY]->thisobject->bbox.height) {
-	    objectdeselect();	
+	    unselect_all();	
 	    Wprintf("Could not find appropriate place to insert object");
 	    return;
 	 }
@@ -1126,7 +1133,7 @@ void catmove(XKeyEvent *event)
       linkedlistinsertafter(&(xobjs.userlibs[i].instlist), s1, s2);
    }
 
-   objectdeselect();
+   unselect_all();
    if ((i = is_library(topobject)) >= 0) composelib(LIBRARY + i);
 
    drawarea(NULL, NULL, NULL);
@@ -1173,7 +1180,6 @@ void copycat()
       (*newobj)->pcorner.x = oldobj->pcorner.x;
       (*newobj)->pcorner.y = oldobj->pcorner.y;
       (*newobj)->viewscale = oldobj->viewscale;
-#ifdef SCHEMA
       /* don't attach the same schematic. . . */
       (*newobj)->symschem = NULL;
 
@@ -1194,6 +1200,9 @@ void copycat()
 	    case XC_STRING:
 	       newops->parameter.string = stringcopy(ops->parameter.string);
 	       break;
+	    case XC_EXPR:
+	       newops->parameter.expr = strdup(ops->parameter.expr);
+	       break;
 	 }
       }
 
@@ -1204,7 +1213,6 @@ void copycat()
       (*newobj)->valid = False;
       (*newobj)->traversed = False;
       (*newobj)->devname = NULL;
-#endif
       (*newobj)->hidden = False;
 
       /* copy over all the elements of the original object */
@@ -1213,7 +1221,7 @@ void copycat()
       (*newobj)->plist = (genericptr *)malloc(sizeof(genericptr));
 
       for (i = 0; i < oldobj->parts; i++) {
-	 switch((*(oldobj->plist + i))->type) {
+	 switch(ELEMENTTYPE(*(oldobj->plist + i))) {
 	    case(PATH): {
 	       register pathptr *npath, cpath = TOPATH(oldobj->plist + i);
 	       register genericptr *gpart;
@@ -1226,7 +1234,7 @@ void copycat()
                (*npath)->passed = NULL;
 	       for (gpart = cpath->plist; gpart < cpath->plist + cpath->parts;
 			gpart++) {
-		  switch((*gpart)->type){
+		  switch(ELEMENTTYPE(*gpart)){
 		     case ARC: {
 			arcptr copyarc = TOARC(gpart);
 			arcptr *newarc;
@@ -1287,12 +1295,10 @@ void copycat()
 	       (*nlabel)->justify = clabel->justify;
 	       (*nlabel)->string = stringcopy(clabel->string);
 	       (*nlabel)->passed = NULL;
-#ifdef SCHEMA
 	       (*nlabel)->pin = clabel->pin;
-#endif
                } break;
       
-	    case(OBJECT): {
+	    case(OBJINST): {
 	       register objinstptr *ninst, cinst;
 
 	       cinst = TOOBJINST(oldobj->plist + i);
@@ -1312,11 +1318,10 @@ void copycat()
 	       (*ninst)->bbox.width = cinst->thisobject->bbox.width;
 	       (*ninst)->bbox.height = cinst->thisobject->bbox.height;
 
-#ifdef SCHEMA
 	       (*ninst)->schembbox = NULL;
 
 	       /* here---deal with attached schematics */
-#endif
+
                } break;
          }
 	 (*newobj)->parts++;
@@ -1327,7 +1332,7 @@ void copycat()
    addtoinstlist(USERLIB - LIBRARY, *newobj, FALSE);
 
    composelib(USERLIB);
-   objectdeselect();
+   unselect_all();
 
    if (areastruct.topinstance == xobjs.libtop[USERLIB])
       drawarea(NULL, NULL, NULL);
@@ -1359,7 +1364,7 @@ void catbutton(u_int mode, XButtonEvent *event)
 	
       for (libobj = (objinstptr *)topobject->plist; libobj <
 		(objinstptr *)topobject->plist + topobject->parts; libobj++) {
-	 if ((*libobj)->type == OBJECT) {
+	 if (IS_OBJINST(*libobj)) {
 
  	    ocentx = (*libobj)->position.x + (*libobj)->bbox.lowerleft.x
 	        + ((*libobj)->bbox.width >> 1);
@@ -1377,7 +1382,6 @@ void catbutton(u_int mode, XButtonEvent *event)
 
 	       /* setup to move object around and draw the selected object */
 
-#ifdef SCHEMA
 	       if (eventmode == ASSOC_MODE) {
 
 	          /* revert to old page */
@@ -1390,9 +1394,7 @@ void catbutton(u_int mode, XButtonEvent *event)
 		  catreturn();
 		  eventmode = PRESS_MODE;
 	       }
-	       else
-#endif
-               if (event->button == Button1) {
+	       else if (event->button == Button1) {
 
 	          /* revert to old page */
 
@@ -1461,14 +1463,18 @@ void catbutton(u_int mode, XButtonEvent *event)
                      eventmode = PRESS_MODE;
 	          }
 #ifdef TCL_WRAPPER
+		  /* fprintf(stderr, "Creating event handler for xctk_drag: "); */
+		  /* printeventmode();		*/
                   Tk_CreateEventHandler(areastruct.area, PointerMotionMask,
 			(Tk_EventProc *)xctk_drag, NULL);
 #endif
+	 	  register_for_undo(XCF_Library_Pop, UNDO_MORE, areastruct.topinstance,
+				areastruct.selectlist, areastruct.selects);
                   catreturn();
                }
 
-               /* If button2 was pressed, select and stay in the catalog. */
-	       /* Could just do "objectselect" here, but would not cover   */
+               /* If button2 was pressed, select and stay in the catalog.  */
+	       /* Could just do "select_element" here, but would not cover */
 	       /* the entire area in the directory surrounding the object. */
 
 	       else {
@@ -1540,11 +1546,9 @@ void startcatalog(xcWidget w, pointertype libmod, caddr_t nulldata)
       else
          eventmode = FONTCAT2_MODE;
    }
-#ifdef SCHEMA
    else if (eventmode == ASSOC_MODE) {
       XDefineCursor (dpy, areastruct.areawin, CROSS);
    }
-#endif
    else if (libmod == PAGELIB || libmod == LIBLIB) {
       XDefineCursor (dpy, areastruct.areawin, CROSS);
       eventmode = CATALOG_MODE;
