@@ -204,7 +204,7 @@ menustruct PolyMenu[] = {
 
 menustruct FilterMenu[] = {
 	{"Labels", setaction(selectfilter, Number(LABEL))},
-	{"Objects", setaction(selectfilter, Number(OBJECT))},
+	{"Objects", setaction(selectfilter, Number(OBJINST))},
 	{"Polygons", setaction(selectfilter, Number(POLYGON))},
 	{"Arcs", setaction(selectfilter, Number(ARC))},
 	{"Splines", setaction(selectfilter, Number(SPLINE))},
@@ -230,6 +230,11 @@ menustruct RotateMenu[] = {
 	{"Rotate CCW 1", action(startrotate, Number(-1))},
 };
 
+menustruct LineMenu[] = {
+	{"Global Linewidth", action(getwirewidth, NULL)},
+	{"Wire Linewidth", action(getwwidth, NULL)},
+}
+
 menustruct GridMenu[] = {
 	{"Grid", setaction(toggle, &areastruct.gridon)},
 	{"Axes", setaction(toggle, &areastruct.axeson)},
@@ -312,15 +317,17 @@ menustruct OptionMenu[] = {
 #endif
 	{"Grid", submenu(GridMenu)},
 	{"Snap-to", submenu(SnapMenu)},
-	{"Global Linewidth", action(getwirewidth, NULL)},
+	{"Linewidth", submenu(LineMenu)},
 	{"Elements", submenu(PolyMenu)},
 	{" ", noaction},
 	{"Help!", action(starthelp, NULL)},
 };
 
 menustruct EditMenu[] = {
+	{"Undo (u)", action(undo_call, NULL)},
+	{"Redo (U)", action(redo_call, NULL)},
+	{" ", noaction},
 	{"Delete (d)", action(startdelete, NULL)},
-	{"Undelete (u)", action(xc_undelete, Number(DRAW))},
 	{"Copy (c)", action(startcopy, NULL)},
 	{"Edit (e)", action(startedit, NULL)},
 	{"Rotate/Flip", submenu(RotateMenu)},
