@@ -504,6 +504,11 @@ extern Boolean dospecial(void);
 
 /* from netlist.c: */
 
+#ifdef TCL_WRAPPER
+extern Tcl_Obj *tclglobals(objinstptr);
+extern Tcl_Obj *tcltoplevel(objinstptr);
+#endif
+
 extern Boolean RemoveFromNetlist(objectptr, genericptr);
 extern labelptr NetToLabel(int, objectptr);
 extern void NameToPosition(objinstptr, labelptr, XPoint *);
@@ -758,6 +763,7 @@ extern void quit(xcWidget, caddr_t);
 extern void resizetoolbar();
 extern void writescalevalues(char *, char *, char *);
 #ifdef TCL_WRAPPER
+extern Tcl_Obj *Tcl_NewHandleObj();
 extern int Tcl_GetHandleFromObj(Tcl_Interp *, Tcl_Obj *, void **);
 #else
 extern void updatetext(xcWidget, xcWidgetList, caddr_t);
